from django.conf.urls import url
from example_app.views import (
    OrderListCreateEndpoint, OrderRetrieveUpdateDestroyEndpoint, OrderRetrieveUpdateEndpoint,
    OrderCreateWithGenericEndpoint, OrderListWithGenericEndpoint, OrderRetrieveWithGenericEndpoint,
    OrderUpdateWithGenericEndpoint, OrderCreateWithMixinEndpoint, OrderUpdateWithMixinEndpoint,
    OrderListWithoutReadSerializerEndpoint, OrderListWithMixinEndpoint, OrderRetrieveWithMixinEndpoint,
    OrderViewset
)


urlpatterns = [
    url(
        r'^orders-list-without-read-serializer/$',
        OrderListWithoutReadSerializerEndpoint.as_view(),
        name='list_without_request_serializer'
    ),
    url(
        r'^orders/$',
        OrderListCreateEndpoint.as_view(),
        name='list_create'
    ),
    url(
        r'^orders/(?P<pk>[0-9]+)$',
        OrderRetrieveUpdateDestroyEndpoint.as_view(),
        name='retrieve_update_destroy'
    ),
    url(
        r'^orders-retrieve-update/(?P<pk>[0-9]+)$',
        OrderRetrieveUpdateEndpoint.as_view(),
        name='retrieve_update'
    ),
    url(
        r'^orders-list-generic-view/$',
        OrderListWithGenericEndpoint.as_view(),
        name='list'
    ),
    url(
        r'^orders-retrieve-generic-view/(?P<pk>[0-9]+)$',
        OrderRetrieveWithGenericEndpoint.as_view(),
        name='retrieve'
    ),
    url(
        r'^orders-create-generic-view/$',
        OrderCreateWithGenericEndpoint.as_view(),
        name='create'
    ),
    url(
        r'^orders-update-generic-view/(?P<pk>[0-9]+)$',
        OrderUpdateWithGenericEndpoint.as_view(),
        name='update'
    ),
    url(
        r'^orders-list-mixin/$',
        OrderListWithMixinEndpoint.as_view(),
        name='list_mixin'
    ),
    url(
        r'^orders-retrieve-mixin/(?P<pk>[0-9]+)$',
        OrderRetrieveWithMixinEndpoint.as_view(),
        name='retrieve_mixin'
    ),
    url(
        r'^orders-create-mixin/$',
        OrderCreateWithMixinEndpoint.as_view(),
        name='create_mixin'
    ),
    url(
        r'^orders-update-mixin/(?P<pk>[0-9]+)$',
        OrderUpdateWithMixinEndpoint.as_view(),
        name='update_mixin'
    ),

    url(
        r'^orders-viewset/$',
        OrderViewset.as_view({
            'get': 'list',
            'post': 'create',
        }),
        name='viewset_list_create'
    ),
    url(
        r'^orders-viewset/(?P<pk>[0-9]+)$',
        OrderViewset.as_view({
            'get': 'retrieve',
            'put': 'update',
            'patch': 'partial_update',
            'delete': 'destroy',
        }),
        name='viewset_retrieve_update_destroy'
    ),
]



from rest_framework import viewsets

from drf_crud_proxy.mixins import CreateModelMixin, UpdateModelMixin, ListModelMixin, RetrieveModelMixin
from drf_crud_proxy.proxy import CRUDProxy


class GenericViewSet(CRUDProxy, viewsets.GenericViewSet):
    pass


class ModelViewSet(CRUDProxy, CreateModelMixin, UpdateModelMixin, ListModelMixin, RetrieveModelMixin, viewsets.ModelViewSet):
    pass


class ReadOnlyModelViewSet(CRUDProxy, ListModelMixin, RetrieveModelMixin, viewsets.ReadOnlyModelViewSet):
    pass



from rest_framework import generics

from drf_crud_proxy.mixins import CreateModelMixin, UpdateModelMixin, ListModelMixin, RetrieveModelMixin
from drf_crud_proxy.proxy import CRUDProxy


class CreateAPIView(CRUDProxy, CreateModelMixin, generics.CreateAPIView):
    pass


class UpdateAPIView(CRUDProxy, UpdateModelMixin, generics.UpdateAPIView):
    pass


class ListAPIView(CRUDProxy, ListModelMixin, generics.ListAPIView):
    pass


class RetrieveAPIView(CRUDProxy, RetrieveModelMixin, generics.RetrieveAPIView):
    pass


class ListCreateAPIView(CRUDProxy, CreateModelMixin, ListModelMixin, generics.ListCreateAPIView):
    pass


class RetrieveDestroyAPIView(CRUDProxy, RetrieveModelMixin, generics.RetrieveDestroyAPIView):
    pass


class RetrieveUpdateAPIView(CRUDProxy, UpdateModelMixin, RetrieveModelMixin, generics.RetrieveUpdateAPIView):
    pass


class RetrieveUpdateDestroyAPIView(CRUDProxy, UpdateModelMixin, RetrieveModelMixin, generics.RetrieveUpdateDestroyAPIView):
    pass



# Generated by Django 2.1 on 2018-08-20 07:39

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('customer', models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='OrderedProduct',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('quantity', models.IntegerField(default=1)),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ordered_products', to='example_app.Order')),
            ],
        ),
        migrations.CreateModel(
            name='Product',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('price', models.DecimalField(decimal_places=2, max_digits=5)),
            ],
        ),
        migrations.AddField(
            model_name='orderedproduct',
            name='product',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='example_app.Product'),
        ),
    ]



from django.contrib.auth import get_user_model
from model_mommy import mommy
from rest_framework.test import APITestCase, APIClient

from example_app.models import Order
from example_app.serializers import OrderedProductDetailsSerializer


class BaseTestCase(APITestCase):
    def setUp(self):
        self.products = mommy.make('example_app.Product', _quantity=3)

        user_model = get_user_model()

        self.user_email = 'user'
        self.user_password = 'the_password'
        self.user = user_model.objects.create_user(
            username=self.user_email, password=self.user_password)

        self.auth_client = APIClient()
        self.auth_client.force_authenticate(self.user)


class TestListRequestSuccess(object):
    def test_list_request_success(self):
        orders = mommy.make('example_app.Order', _quantity=3)
        for order in orders:
            mommy.make('example_app.OrderedProduct', order=order, _quantity=2)

        response = self.auth_client.get(self.view_url, format='json')
        self.assertEqual(response.status_code, 200)
        self.assertCountEqual(
            response.data, self.list_serializer_class(orders, many=True).data)


class TestRetrieveRequestSuccess(object):
    def test_list_request_success(self):
        order = mommy.make('example_app.Order')
        mommy.make('example_app.OrderedProduct', order=order, _quantity=2)
        response = self.auth_client.get(self.view_url, format='json')
        self.assertEqual(response.status_code, 200)
        self.assertCountEqual(
            response.data, self.retrieve_serializer_class(order).data)


class TestCreateRequestSuccess(object):
    def test_create_request_success(self):
        data = {
            'customer': 100,
            'ordered_products': [
                {
                    'quantity': 1,
                    'product': self.products[0].id,
                },
                {
                    'quantity': 2,
                    'product': self.products[1].id,
                },
            ],
        }
        response = self.auth_client.post(self.view_url, data, format='json')
        self.assertEqual(response.status_code, 201)
        order = Order.objects.get(id=response.data['id'])
        self.assertEqual(response.data, self.create_out_serializer_class(order).data)
        self.assertEqual(order.customer, data['customer'])

        for ordered_product_dict in data['ordered_products']:
            ordered_product = order.ordered_products.filter(product__id=ordered_product_dict['product']).first()
            self.assertIsNotNone(ordered_product)
            self.assertEqual(ordered_product.quantity, ordered_product_dict['quantity'])


class TestUpdateRequestSuccess(object):
    def test_update_request_success(self):
        data = {
            'customer': 2,
            'ordered_products': [
                {
                    'quantity': 10,
                    'product': self.products[0].id,
                },
                {
                    'quantity': 20,
                    'product': self.products[1].id,
                },
            ]
        }
        response = self.auth_client.put(self.view_url, data, format='json')
        self.object.refresh_from_db()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data, self.update_out_serializer_class(self.object).data)
        self.assertEqual(self.object.customer, data['customer'])

        for ordered_product_dict in data['ordered_products']:
            ordered_product = self.object.ordered_products.filter(
                product__id=ordered_product_dict['product']).first()
            self.assertIsNotNone(ordered_product)
            self.assertEqual(ordered_product.quantity, ordered_product_dict['quantity'])

    def test_partial_update_customer_request_success(self):
        old_ordered_products = OrderedProductDetailsSerializer(
            self.object.ordered_products.all(), many=True).data

        data = {
            'customer': 2,
        }

        response = self.auth_client.patch(self.view_url, data, format='json')
        self.assertEqual(response.status_code, 200)
        self.object.refresh_from_db()
        self.assertEqual(response.data, self.update_out_serializer_class(self.object).data)
        self.assertEqual(self.object.customer, data['customer'])
        self.assertCountEqual(
            OrderedProductDetailsSerializer(self.object.ordered_products.all(), many=True).data,
            old_ordered_products)

    def test_partial_update_ordered_products_request_success(self):
        old_customer = self.object.customer
        data = {
            'ordered_products': [
                {
                    'quantity': 10,
                    'product': self.products[2].id,
                },
            ]
        }
        response = self.auth_client.patch(self.view_url, data, format='json')
        self.assertEqual(response.status_code, 200)
        self.object.refresh_from_db()
        self.assertEqual(response.data, self.update_out_serializer_class(self.object).data)
        for ordered_product_dict in data['ordered_products']:
            ordered_product = self.object.ordered_products.filter(
                product__id=ordered_product_dict['product']).first()
            self.assertIsNotNone(ordered_product)
            self.assertEqual(ordered_product.quantity, ordered_product_dict['quantity'])
        self.assertEqual(self.object.customer, old_customer)



from rest_framework.generics import GenericAPIView

from drf_crud_proxy import generics, viewsets, mixins
from drf_crud_proxy.proxy import CRUDProxy
from .models import Order
from .serializers import OrderCreateSerializer, OrderListSerializer


class OrderBaseEndpoint(object):
    queryset = Order.objects.all()
    request_serializer_class = OrderCreateSerializer
    response_serializer_class = OrderListSerializer


class OrderListCreateEndpoint(OrderBaseEndpoint, generics.ListCreateAPIView):
    pass


class OrderListWithoutReadSerializerEndpoint(generics.ListAPIView):
    queryset = Order.objects.all()
    serializer_class = OrderListSerializer


class OrderRetrieveUpdateDestroyEndpoint(OrderBaseEndpoint, generics.RetrieveUpdateDestroyAPIView):
    pass


class OrderRetrieveUpdateEndpoint(OrderBaseEndpoint, generics.RetrieveUpdateAPIView):
    pass


class OrderCreateWithGenericEndpoint(OrderBaseEndpoint, generics.CreateAPIView):
    pass


class OrderUpdateWithGenericEndpoint(OrderBaseEndpoint, generics.UpdateAPIView):
    pass


class OrderListWithGenericEndpoint(OrderBaseEndpoint, generics.ListAPIView):
    pass


class OrderRetrieveWithGenericEndpoint(OrderBaseEndpoint, generics.RetrieveAPIView):
    pass


class OrderCreateWithMixinEndpoint(CRUDProxy, OrderBaseEndpoint, GenericAPIView, mixins.CreateModelMixin):
    pass

    def post(self, request, *args, **kwargs):
        return self.create(request, *args, **kwargs)


class OrderUpdateWithMixinEndpoint(CRUDProxy, OrderBaseEndpoint, GenericAPIView, mixins.UpdateModelMixin):
    pass

    def put(self, request, *args, **kwargs):
        return self.update(request, *args, **kwargs)

    def patch(self, request, *args, **kwargs):
        return self.partial_update(request, *args, **kwargs)


class OrderListWithMixinEndpoint(CRUDProxy, OrderBaseEndpoint, GenericAPIView, mixins.ListModelMixin):
    pass

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)


class OrderRetrieveWithMixinEndpoint(CRUDProxy, OrderBaseEndpoint, GenericAPIView, mixins.RetrieveModelMixin):
    pass

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)


class OrderViewset(OrderBaseEndpoint, viewsets.ModelViewSet):
    pass



from django.db import models


class Order(models.Model):
    customer = models.IntegerField()

    @property
    def total_price(self):
        return sum([
            o.product.price * o.quantity
            for o in self.ordered_products.select_related('product').all()
        ])


class Product(models.Model):
    name = models.CharField(max_length=100)
    price = models.DecimalField(decimal_places=2, max_digits=5)


class OrderedProduct(models.Model):
    order = models.ForeignKey(Order, related_name='ordered_products', on_delete=models.CASCADE)
    product = models.ForeignKey(Product, on_delete=models.CASCADE)
    quantity = models.IntegerField(default=1)



import os
import re

from setuptools import setup, find_packages


def get_version(*file_paths):
    """
    Extract the version string from the file at the given relative path fragments.
    """
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


VERSION = get_version('drf_crud_proxy', '__init__.py')


setup(
    name='drf-crud-proxy',
    version=VERSION,
    description="""Generic views, viewsets and mixins that extend the ones provided by Django REST Framework, adding separated serializers for read and write operations.""",
    author='attevaltonen',
    author_email='atte.hj.valtonen@gmail.com',
    url='https://github.com/attevaltonen/drf-crud-proxy',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Django>=2.0,<2.2',
        'djangorestframework>=3.8'
    ],
    license='MIT',
    zip_safe=False,
    keywords='Django REST Framework, Serializers, REST, API, Django',
    test_suite='tox',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: Finnish',
        'Programming Language :: Python :: 3.7',
    ],
)



"""
Test utilities.

Since py.test discourages putting __init__.py into test directory (i.e. making tests a package)
one cannot import from anywhere under tests folder. However, some utility classes/methods might
be useful in multiple test modules (i.e. factoryboy factories, base test classes). So this package
is the place to put them.
"""



from rest_framework import serializers
from .models import Order, OrderedProduct, Product


class OrderedProductCreateSerializer(serializers.ModelSerializer):

    class Meta:
        model = OrderedProduct
        fields = ('id', 'quantity', 'product')


class OrderCreateSerializer(serializers.ModelSerializer):
    ordered_products = OrderedProductCreateSerializer(many=True)

    class Meta:
        model = Order
        fields = ('id', 'customer', 'ordered_products', 'total_price')

    def create(self, validated_data):
        ordered_products_dict_list = validated_data.pop('ordered_products')
        instance = Order.objects.create(**validated_data)

        for ordered_product_dict in ordered_products_dict_list:
            instance.ordered_products.create(**ordered_product_dict)

        return instance

    def update(self, instance, validated_data):
        ordered_products_dict_list = validated_data.pop('ordered_products', None)

        instance.customer = validated_data.pop('customer', instance.customer)
        instance.save()

        if ordered_products_dict_list is not None:
            instance.ordered_products.all().delete()
            for ordered_product_dict in ordered_products_dict_list:
                instance.ordered_products.create(**ordered_product_dict)

        return instance


class ProductSerializer(serializers.ModelSerializer):
    class Meta:
        model = Product
        fields = ('id', 'name', 'price')


class OrderedProductDetailsSerializer(serializers.ModelSerializer):
    product = ProductSerializer()

    class Meta:
        model = OrderedProduct
        fields = ('id', 'quantity', 'product')


class OrderListSerializer(serializers.ModelSerializer):
    ordered_products = OrderedProductDetailsSerializer(many=True)

    class Meta:
        model = Order
        fields = ('id', 'customer', 'ordered_products', 'total_price')
